"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _logger = require("../logger");

function _getRequireWildcardCache() { if (typeof WeakMap !== "function") return null; var cache = new WeakMap(); _getRequireWildcardCache = function () { return cache; }; return cache; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

class FeaturesManager {
  constructor() {
    _defineProperty(this, "open", async (currentFeatureGuid, featureGuid, moduleGuid, featurePath, dbContext, setCurrentFeatureState) => {
      if (currentFeatureGuid && currentFeatureGuid !== featureGuid) {
        this.features[currentFeatureGuid].stopEmission = true;
      }

      let loadedFeatureInstance = this.features[featureGuid];

      if (!loadedFeatureInstance) {
        try {
          const {
            default: LoadedFeature
          } = await Promise.resolve(`${featurePath}`).then(s => _interopRequireWildcard(require(s)));
          loadedFeatureInstance = new LoadedFeature(featureGuid, moduleGuid, {
            dbContext
          });
          loadedFeatureInstance.on('state', setCurrentFeatureState);
          this.features[featureGuid] = loadedFeatureInstance;
          loadedFeatureInstance.stopEmission = false;
        } catch (error) {
          (0, _logger.logError)(`features-manager:open ERROR: ${error.message}. path:  ${featurePath}`, error);
          return {
            error: error.message
          };
        }
      }

      loadedFeatureInstance.stopEmission = false;
      return loadedFeatureInstance.getState();
    });

    _defineProperty(this, "close", (featureGuid, setCurrentFeatureState) => {
      if (this.features[featureGuid]) {
        this.features[featureGuid].off('state', setCurrentFeatureState);
        this.features[featureGuid].close();
        delete this.features[featureGuid];
      }

      return null;
    });

    _defineProperty(this, "closeAll", () => {
      const loadedFeatures = Object.keys(this.features);
      const loadedFeaturesCount = loadedFeatures.length;

      for (let index = 0; index < loadedFeaturesCount; index += 1) {
        this.close(loadedFeatures[index]);
      }
    });

    _defineProperty(this, "handleAction", action => {
      const loadedFeature = this.features[action.featureGuid];

      if (loadedFeature) {
        loadedFeature.execute(action);
        return;
      }

      throw new Error(`Feature ${action.featureGuid} not found`);
    });

    this.features = {};
  }

}

var _default = FeaturesManager;
exports.default = _default;