"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _path = _interopRequireDefault(require("path"));

var _fs = _interopRequireDefault(require("fs"));

var _logger = require("../logger");

var _customItem = _interopRequireWildcard(require("../custom-item"));

var _commandsSlice = _interopRequireWildcard(require("./store/commands-slice"));

var _featuresSlice = _interopRequireWildcard(require("./store/features-slice"));

var _featuresManager = _interopRequireDefault(require("./features-manager"));

function _getRequireWildcardCache() { if (typeof WeakMap !== "function") return null; var cache = new WeakMap(); _getRequireWildcardCache = function () { return cache; }; return cache; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

class ModuleBase extends _customItem.default {
  constructor(guid, ownerGuid, options = {}) {
    super(guid, ownerGuid, options);

    _defineProperty(this, "setCurrentFeatureStateAndEmit", currentFeatureState => {
      this.dispatch((0, _featuresSlice.setCurrentFeature)(currentFeatureState));
      this.emitState();
    });

    this.featuresRootDir = options.featuresRootDir;
    this.store = this.store.injectReducer('commands', _commandsSlice.default);
    this.store = this.store.injectReducer('features', _featuresSlice.default);
    this.featuresManager = new _featuresManager.default();
    this.store.dispatch((0, _commandsSlice.setForeground)((0, _commandsSlice.buildForeground)(guid)));
    this.store.dispatch((0, _commandsSlice.setBackground)((0, _commandsSlice.buildBackground)(guid)));
    this.store.dispatch((0, _commandsSlice.setOpenFeature)((0, _commandsSlice.buildOpenFeature)(guid)));
    this.store.dispatch((0, _commandsSlice.setCloseFeature)((0, _commandsSlice.buildCloseFeature)(guid)));
    this.store.dispatch((0, _commandsSlice.setStartEmission)((0, _commandsSlice.buildStartEmission)(guid)));
    this.store.dispatch((0, _commandsSlice.setStopEmission)((0, _commandsSlice.buildStopEmission)(guid)));

    if (options.featuresRootDir) {
      const featuresFileName = _path.default.join(options.featuresRootDir, 'features.json');

      if (_fs.default.existsSync(featuresFileName)) {
        const availableFeatures = JSON.parse(_fs.default.readFileSync(featuresFileName));
        this.store.dispatch((0, _featuresSlice.setAvailableFeatures)(availableFeatures.map(availableFeature => ({ ...availableFeature,
          moduleGuid: ownerGuid
        }))));
      } else {
        (0, _logger.logError)(`File not found ${featuresFileName}`);
      }
    }
  }

  getPublicState() {
    const result = { ...super.getPublicState(),
      commands: this.useSelector(_commandsSlice.selectCommandsArray),
      availableFeatures: this.useSelector(_featuresSlice.selectAvailableFeaturesArray),
      currentFeature: this.useSelector(_featuresSlice.selectCurrentFeature)
    };
    return result;
  }

  async execute(action) {
    if (action.sink === 'FEATURE') {
      this.featuresManager.handleAction(action);
      return;
    }

    if (action.type === _commandsSlice.actionTypes.FOREGROUND || action.type === _commandsSlice.actionTypes.BACKGROUND) {
      try {
        this.dispatch(_customItem.statusSlice.actions.setIsBusy(true));
        const isForeground = action.type === _commandsSlice.actionTypes.FOREGROUND;
        this.dispatch((0, _commandsSlice.setForegroundIsEnabled)(!isForeground));
        this.dispatch((0, _commandsSlice.setBackgroundIsEnabled)(isForeground));
        this.dispatch(_customItem.statusSlice.actions.setIsForeground(isForeground));
      } finally {
        this.dispatch(_customItem.statusSlice.actions.setIsBusy(false));
        this.emitState(true);
      }
    } // eslint-disable-next-line default-case


    switch (action.type) {
      case _commandsSlice.actionTypes.START_EMISSION:
        this.stopEmission = false;
        break;

      case _commandsSlice.actionTypes.STOP_EMISSION:
        this.stopEmission = true;
        break;

      case _commandsSlice.actionTypes.OPEN_FEATURE:
        try {
          this.dispatch(_customItem.statusSlice.actions.setIsBusy(true));
          const currentFeatureGuid = this.useSelector(_featuresSlice.selectCurrentFeatureGuid);
          this.dispatch((0, _featuresSlice.setCurrentFeature)((await this.featuresManager.open(currentFeatureGuid, action.featureGuid, this.getPublicState().guid, _path.default.join(this.featuresRootDir, action.folderPath), action.dbContext === 'shared' ? this.dbContext : null, this.setCurrentFeatureStateAndEmit))));
        } finally {
          this.dispatch(_customItem.statusSlice.actions.setIsBusy(false));
          this.emitState(true);
        }

        break;

      case _commandsSlice.actionTypes.CLOSE_FEATURE:
        try {
          this.dispatch(_customItem.statusSlice.actions.setIsBusy(true));
          this.dispatch((0, _featuresSlice.setCurrentFeature)((await this.featuresManager.close(action.featureGuid, this.setCurrentFeatureStateAndEmit))));
        } finally {
          this.dispatch(_customItem.statusSlice.actions.setIsBusy(false));
          this.emitState();
        }

        break;
    }
  }

  unload() {
    this.featuresManager.closeAll();

    if (this.dbContextIsLocal) {
      this.dbContext.close();
    }
  }

}

var _default = ModuleBase;
exports.default = _default;